@@ -13,20 +13,166 @@ LICENSE END */
     [PlatformWindow] trait used by the generated code and the run-time to change
     aspects of windows on the screen.
 */
+use corelib::component::ComponentRc;
 use corelib::items::PointerEventButton;
+use corelib::layout::Orientation;
 use sixtyfps_corelib as corelib;
 
 use corelib::graphics::Point;
 use corelib::input::{InternalKeyCode, KeyEvent, KeyEventType, KeyboardModifiers, MouseEvent};
-use corelib::window::*;
 use corelib::SharedString;
-use std::cell::RefCell;
+use corelib::{window::*, Color};
+use std::cell::{Cell, RefCell};
 use std::rc::{Rc, Weak};
 use winit::event::WindowEvent;
 
 #[cfg(not(target_arch = "wasm32"))]
 use winit::platform::run_return::EventLoopExtRunReturn;
 
+pub trait WinitWindow: PlatformWindow {
+    fn runtime_window(&self) -> Rc<corelib::window::Window>;
+    fn currently_pressed_key_code(&self) -> &Cell<Option<winit::event::VirtualKeyCode>>;
+    fn current_keyboard_modifiers(&self) -> &Cell<KeyboardModifiers>;
+    fn draw(self: Rc<Self>);
+    fn with_window_handle(&self, callback: &mut dyn FnMut(&winit::window::Window));
+    fn constraints(&self) -> (corelib::layout::LayoutInfo, corelib::layout::LayoutInfo);
+    fn set_constraints(
+        &self,
+        constraints: (corelib::layout::LayoutInfo, corelib::layout::LayoutInfo),
+    );
+    fn set_background_color(&self, color: Color);
+    fn set_icon(&self, icon: corelib::graphics::Image);
+
+    fn apply_constraints(
+        &self,
+        constraints_horizontal: corelib::layout::LayoutInfo,
+        constraints_vertical: corelib::layout::LayoutInfo,
+    ) {
+        self.with_window_handle(&mut |winit_window| {
+            // If we're in fullscreen state, don't try to resize the window but maintain the surface
+            // size we've been assigned to from the windowing system. Weston/Wayland don't like it
+            // when we create a surface that's bigger than the screen due to constraints (#532).
+            if winit_window.fullscreen().is_some() {
+                return;
+            }
+
+            if (constraints_horizontal, constraints_vertical) != self.constraints() {
+                let min_width = constraints_horizontal.min.min(constraints_horizontal.max);
+                let min_height = constraints_vertical.min.min(constraints_vertical.max);
+                let max_width = constraints_horizontal.max.max(constraints_horizontal.min);
+                let max_height = constraints_vertical.max.max(constraints_vertical.min);
+
+                winit_window.set_min_inner_size(if min_width > 0. || min_height > 0. {
+                    Some(winit::dpi::LogicalSize::new(min_width, min_height))
+                } else {
+                    None
+                });
+                winit_window.set_max_inner_size(if max_width < f32::MAX || max_height < f32::MAX {
+                    Some(winit::dpi::LogicalSize::new(
+                        max_width.min(65535.),
+                        max_height.min(65535.),
+                    ))
+                } else {
+                    None
+                });
+                self.set_constraints((constraints_horizontal, constraints_vertical));
+
+                #[cfg(target_arch = "wasm32")]
+                {
+                    // set_max_inner_size / set_min_inner_size don't work on wasm, so apply the size manually
+                    let existing_size = winit_window.inner_size();
+                    if !(min_width..=max_width).contains(&(existing_size.width as f32))
+                        || !(min_height..=max_height).contains(&(existing_size.height as f32))
+                    {
+                        let new_size = winit::dpi::LogicalSize::new(
+                            existing_size
+                                .width
+                                .min(max_width.ceil() as u32)
+                                .max(min_width.ceil() as u32),
+                            existing_size
+                                .height
+                                .min(max_height.ceil() as u32)
+                                .max(min_height.ceil() as u32),
+                        );
+                        winit_window.set_inner_size(new_size);
+                    }
+                }
+            }
+        });
+    }
+
+    fn apply_window_properties(
+        &self,
+        window_item: core::pin::Pin<&sixtyfps_corelib::items::WindowItem>,
+    ) {
+        let background = window_item.background();
+        let title = window_item.title();
+        let no_frame = window_item.no_frame();
+        let icon = window_item.icon();
+        let width = window_item.width();
+        let height = window_item.height();
+
+        self.set_background_color(background);
+        self.set_icon(icon);
+
+        let mut size: winit::dpi::LogicalSize<f64> = Default::default();
+        self.with_window_handle(&mut |winit_window| {
+            winit_window.set_title(&title);
+            if no_frame && winit_window.fullscreen().is_none() {
+                winit_window.set_decorations(false);
+            } else {
+                winit_window.set_decorations(true);
+            }
+            size = winit_window.inner_size().to_logical(winit_window.scale_factor() as f64);
+        });
+
+        let mut must_resize = false;
+        let mut resizable = None;
+        let mut w = width;
+        let mut h = height;
+        if (size.width as f32 - w).abs() < 1. || (size.height as f32 - h).abs() < 1. {
+            return;
+        }
+        if w <= 0. || h <= 0. {
+            if let Some(component_rc) = self.runtime_window().try_component() {
+                let component = ComponentRc::borrow_pin(&component_rc);
+                let hor_info = component.as_ref().layout_info(Orientation::Horizontal);
+                if w <= 0. {
+                    w = hor_info.preferred_bounded();
+                    must_resize = true;
+                }
+                let ver_info = component.as_ref().layout_info(Orientation::Vertical);
+                if h <= 0. {
+                    h = ver_info.preferred_bounded();
+                    must_resize = true;
+                }
+                resizable = Some(hor_info.min != hor_info.max && ver_info.min != ver_info.max);
+            }
+        };
+        if w > 0. {
+            size.width = w as _;
+        }
+        if h > 0. {
+            size.height = h as _;
+        }
+        self.with_window_handle(&mut |winit_window| {
+            // If we're in fullscreen state, don't try to resize the window but maintain the surface
+            // size we've been assigned to from the windowing system. Weston/Wayland don't like it
+            // when we create a surface that's bigger than the screen due to constraints (#532).
+            if winit_window.fullscreen().is_none() {
+                winit_window.set_inner_size(size);
+                if let Some(resizable) = resizable {
+                    winit_window.set_resizable(resizable)
+                }
+            }
+        });
+
+        if must_resize {
+            self.runtime_window().set_window_item_geometry(size.width as _, size.height as _)
+        }
+    }
+}
+
 struct NotRunningEventLoop {
     instance: winit::event_loop::EventLoop<CustomEvent>,
     event_loop_proxy: winit::event_loop::EventLoopProxy<CustomEvent>,
@@ -71,7 +217,7 @@ impl<'a> EventLoopInterface for RunningEventLoop<'a> {
 }
 
 thread_local! {
-    static ALL_WINDOWS: RefCell<std::collections::HashMap<winit::window::WindowId, Weak<crate::graphics_window::GraphicsWindow>>> = RefCell::new(std::collections::HashMap::new());
+    static ALL_WINDOWS: RefCell<std::collections::HashMap<winit::window::WindowId, Weak<dyn WinitWindow>>> = RefCell::new(std::collections::HashMap::new());
     static MAYBE_LOOP_INSTANCE: RefCell<Option<NotRunningEventLoop>> = RefCell::new(Some(NotRunningEventLoop::new()));
 }
 
@@ -132,10 +278,7 @@ pub(crate) fn with_window_target<T>(callback: impl FnOnce(&dyn EventLoopInterfac
     }
 }
 
-pub fn register_window(
-    id: winit::window::WindowId,
-    window: Rc<crate::graphics_window::GraphicsWindow>,
-) {
+pub fn register_window(id: winit::window::WindowId, window: Rc<dyn WinitWindow>) {
     ALL_WINDOWS.with(|windows| {
         windows.borrow_mut().insert(id, Rc::downgrade(&window));
     })
@@ -147,7 +290,7 @@ pub fn unregister_window(id: winit::window::WindowId) {
     })
 }
 
-fn window_by_id(id: winit::window::WindowId) -> Option<Rc<crate::graphics_window::GraphicsWindow>> {
+fn window_by_id(id: winit::window::WindowId) -> Option<Rc<dyn WinitWindow>> {
     ALL_WINDOWS.with(|windows| windows.borrow().get(&id).and_then(|weakref| weakref.upgrade()))
 }
 
@@ -186,18 +329,18 @@ fn redraw_all_windows() {
 }
 
 fn process_window_event(
-    window: Rc<crate::graphics_window::GraphicsWindow>,
+    window: Rc<dyn WinitWindow>,
     event: WindowEvent,
     quit_behavior: sixtyfps_corelib::backend::EventLoopQuitBehavior,
     control_flow: &mut winit::event_loop::ControlFlow,
     cursor_pos: &mut Point,
     pressed: &mut bool,
 ) {
-    let runtime_window = window.self_weak.upgrade().unwrap();
+    let runtime_window = window.runtime_window();
     match event {
         WindowEvent::Resized(size) => {
-            let size = size.to_logical(window.scale_factor() as f64);
-            window.set_geometry(size.width, size.height);
+            let size = size.to_logical(runtime_window.scale_factor() as f64);
+            runtime_window.set_window_item_geometry(size.width, size.height);
         }
         WindowEvent::CloseRequested => {
             window.hide();
@@ -216,7 +359,7 @@ fn process_window_event(
             // On Windows, X11 and Wayland sequences like Ctrl+C will send a ReceivedCharacter after the pressed keyboard input event,
             // with a control character. We choose not to forward those but try to use the current key code instead.
             let text: Option<SharedString> = if ch.is_control() {
-                window.currently_pressed_key_code.take().and_then(winit_key_code_to_string)
+                window.currently_pressed_key_code().take().and_then(winit_key_code_to_string)
             } else {
                 Some(ch.to_string().into())
             };
@@ -226,7 +369,7 @@ fn process_window_event(
                 None => return,
             };
 
-            let modifiers = window.current_keyboard_modifiers();
+            let modifiers = window.current_keyboard_modifiers().get();
 
             let mut event = KeyEvent { event_type: KeyEventType::KeyPressed, text, modifiers };
             runtime_window.clone().process_key_input(&event);
@@ -241,7 +384,7 @@ fn process_window_event(
         }
         WindowEvent::KeyboardInput { ref input, .. } => {
             corelib::animations::update_animations();
-            window.currently_pressed_key_code.set(match input.state {
+            window.currently_pressed_key_code().set(match input.state {
                 winit::event::ElementState::Pressed => input.virtual_keycode.clone(),
                 _ => None,
             });
@@ -253,6 +396,7 @@ fn process_window_event(
                     winit::event::VirtualKeyCode::End => Some(InternalKeyCode::End),
                     winit::event::VirtualKeyCode::Back => Some(InternalKeyCode::Back),
                     winit::event::VirtualKeyCode::Delete => Some(InternalKeyCode::Delete),
+                    winit::event::VirtualKeyCode::Tab => Some(InternalKeyCode::Tab),
                     winit::event::VirtualKeyCode::Return
                     | winit::event::VirtualKeyCode::NumpadEnter => Some(InternalKeyCode::Return),
                     winit::event::VirtualKeyCode::Escape => Some(InternalKeyCode::Escape),
@@ -266,7 +410,7 @@ fn process_window_event(
                         winit::event::ElementState::Released => KeyEventType::KeyReleased,
                     },
                     text,
-                    modifiers: window.current_keyboard_modifiers(),
+                    modifiers: window.current_keyboard_modifiers().get(),
                 };
                 runtime_window.process_key_input(&event);
             };
@@ -280,13 +424,13 @@ fn process_window_event(
             let (control, meta) = (state.ctrl(), state.logo());
             let modifiers =
                 KeyboardModifiers { shift: state.shift(), alt: state.alt(), control, meta };
-            window.set_current_keyboard_modifiers(modifiers);
+            window.current_keyboard_modifiers().set(modifiers);
         }
         WindowEvent::CursorMoved { position, .. } => {
             corelib::animations::update_animations();
-            let position = position.to_logical(window.scale_factor() as f64);
+            let position = position.to_logical(runtime_window.scale_factor() as f64);
             *cursor_pos = euclid::point2(position.x, position.y);
-            window.process_mouse_input(MouseEvent::MouseMoved { pos: *cursor_pos });
+            runtime_window.process_mouse_input(MouseEvent::MouseMoved { pos: *cursor_pos });
         }
         WindowEvent::CursorLeft { .. } => {
             // On the html canvas, we don't get the mouse move or release event when outside the canvas. So we have no choice but canceling the event
@@ -294,7 +438,7 @@ fn process_window_event(
             if *pressed {
                 corelib::animations::update_animations();
                 *pressed = false;
-                window.process_mouse_input(MouseEvent::MouseExit);
+                runtime_window.process_mouse_input(MouseEvent::MouseExit);
             }
         }
         WindowEvent::MouseWheel { delta, .. } => {
@@ -304,11 +448,11 @@ fn process_window_event(
                     euclid::point2(lx * 60., ly * 60.)
                 }
                 winit::event::MouseScrollDelta::PixelDelta(d) => {
-                    let d = d.to_logical(window.scale_factor() as f64);
+                    let d = d.to_logical(runtime_window.scale_factor() as f64);
                     euclid::point2(d.x, d.y)
                 }
             };
-            window.process_mouse_input(MouseEvent::MouseWheel { pos: *cursor_pos, delta });
+            runtime_window.process_mouse_input(MouseEvent::MouseWheel { pos: *cursor_pos, delta });
         }
         WindowEvent::MouseInput { state, button, .. } => {
             corelib::animations::update_animations();
@@ -328,11 +472,11 @@ fn process_window_event(
                     MouseEvent::MouseReleased { pos: *cursor_pos, button }
                 }
             };
-            window.process_mouse_input(ev);
+            runtime_window.process_mouse_input(ev);
         }
         WindowEvent::Touch(touch) => {
             corelib::animations::update_animations();
-            let location = touch.location.to_logical(window.scale_factor() as f64);
+            let location = touch.location.to_logical(runtime_window.scale_factor() as f64);
             let pos = euclid::point2(location.x, location.y);
             let ev = match touch.phase {
                 winit::event::TouchPhase::Started => {
@@ -345,11 +489,11 @@ fn process_window_event(
                 }
                 winit::event::TouchPhase::Moved => MouseEvent::MouseMoved { pos },
             };
-            window.process_mouse_input(ev);
+            runtime_window.process_mouse_input(ev);
         }
         WindowEvent::ScaleFactorChanged { scale_factor, new_inner_size: size } => {
             let size = size.to_logical(scale_factor);
-            window.set_geometry(size.width, size.height);
+            runtime_window.set_window_item_geometry(size.width, size.height);
             runtime_window.set_scale_factor(scale_factor as f32);
         }
         _ => {}
@@ -413,7 +557,7 @@ pub fn run(quit_behavior: sixtyfps_corelib::backend::EventLoopQuitBehavior) {
 
                 winit::event::Event::UserEvent(CustomEvent::UpdateWindowProperties(window_id)) => {
                     if let Some(window) = window_by_id(window_id) {
-                        window.self_weak.upgrade().unwrap().update_window_properties();
+                        window.runtime_window().update_window_properties();
                     }
                 }
 
